/************************************************
* EXEDATA - two functions to let you read and	*
* write the data extension on the EXE file.		*
*												*
* 900327 - MCMason: TURBO C v2.01				*
************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <io.h>
#include "exedata.h"

static long pos=0;
static char deNBuf[MAXFNAME+1];

/************************************************
* Err - print error message, and quit			*
************************************************/
void Err(char *s)
	{
	fprintf(stderr,"ERROR: %s\n",s);
	exit(1);
	}

/************************************************
* deSetPos - set the position variable & move	*
* the R/W file pointer to the beginning of the	*
* data extension area.							*
************************************************/
static void deSetPos(FILE *f)
	{
	int r[3];

	/* Don't compute it if it already has value	*/
	if (pos != 0) return;

	/* Read the first three words in EXE header */
	if (fread(r,sizeof(int),3,f) != 3)
		Err("can't read header of EXEDATA.EXE");

	/* Compute extra data starting position */
	pos = 512*(r[2]-1)+r[1]+1;
	}

/************************************************
* deName - set the name of the EXE file			*
*												*
* INP:	pointer to the string holding the name	*
*		of the EXE file.						*
************************************************/
void deName(char *s)
	{
	strncpy(deNBuf,s,MAXFNAME);
	}

/************************************************
* deRead - Read the data extension to the		*
* specified buffer.								*
*												*
* OUT:	TRUE if error has occurred (like data	*
*		extension does not exist), FALSE o/w	*
************************************************/
int deRead(void *buf,int size)
	{
	FILE *f;
	int  retval;

	retval=FALSE;

	/* Open the file */
	if ((f=fopen(deNBuf,"rb"))==NULL)
		Err("can't open EXEDATA.EXE, is it in "
			"the path?");

	/* Set the file position */
	deSetPos(f);

	/* Check if the interesting data is there */
	fseek(f,pos,SEEK_SET);
	if (fread(buf,(size_t)size,1,f) != 1)
		retval=TRUE;

	/* Close the file & return */
	if (fclose(f))
		Err("can't close EXEDATA.EXE");
	return retval;
	}

/************************************************
* deWrite - write the data extension from buff	*
************************************************/
void deWrite(void *buf,int size)
	{
	FILE *f;

	/* Open the file */
	if ((f=fopen(deNBuf,"r+b"))==NULL)
		Err("can't open EXEDATA.EXE");

	deSetPos(f);	/* Set R/W pointer	*/

	/* Move R/W ptr to overlay old data */
	fseek(f,pos,SEEK_SET);

	/* Write the data */
	if (fwrite(buf,(size_t)size,1,f) != 1)
		Err("can't write to EXEDATA.EXE");

	/* Close the file & return */
	if (fclose(f))
		Err("can't close EXEDATA.EXE");
	}
