/************************************************
* RFT - relative file test						*
*												*
* 900324 MCMason - originally written (TC 2.0)	*
************************************************/

#include <stdio.h>
#include <stdlib.h>
#include "files.h"

#define TESTSIZE 25

void main(void)
	{
	int i;
	long j,k;
	RELFILE rf;
	char workbuf[20];

	/* Step 1 - create a file */
	rfDestroy("FILE.TST");
	if (rfCreate("FILE.TST",10))
		{
		puts("ERROR: can't create FILE.TST");
		exit(1);
		}

	/* Step 2 - open the file */
	rf = rfOpen("FILE.TST");
	if (rf == NULL)
		{
		puts("ERROR: can't open FILE.TST");
		exit(2);
		}

	/* Step 3 - write TESTSIZE records to the file */
	for (i=1; i<=TESTSIZE; i++)
		{
		sprintf(workbuf,"Record %03d",i);
		j=rfGetRec(rf);
		if (j==0)
			{
			puts("ERROR: can't get a record");
			exit(3);
			}
		if (rfWrite(rf,j,workbuf))
			{
			puts("ERROR: can't write a record");
			exit(4);
			}
		}

	/* Step 4 - randomly delete records			*/
	for (i=TESTSIZE; i>=1; i--)
		{
		if (rand()%TESTSIZE < i)
			if (rfDelRec(rf,(long)i))
				{
				puts("ERROR: can't delete record");
				exit(5);
				}
		}

	/* Step 5 - randomly add & delete records	*
	 * to 'exercise' the file handler routines	*/
	for (i=0; i<TESTSIZE*3; i++)
		if (rand()%2 == 1)
			{	/* Add a record */
			if (!(j=rfGetRec(rf)))
				puts("ERROR: can't get record");
			sprintf(workbuf,"FilRec %03d",i);
			if (rfWrite(rf,j,workbuf))
				{
				puts("ERROR: can't write record");
				exit(8);
				}
			}
		else
			{	/* Delete a record */
			j=rand()%(rf->LastRec)+1;
			if (rfDelRec(rf,j))
				printf("ERROR: can't delete reco"
					   "rd %ld, status is %d\n",
					   j,rfLastError());
			}

	/* Step 7 - Display the file's free list */
	puts("\n\nThe free list of records is:");
	for (j=rf->NextRec,i=0; j > 0L; )
		{
		printf("% 4ld  ",j);
		if (++i > 12)
			{
			fputc('\n',stdout);
			i=0;
			}
		j=rfRecStat(rf,j);
		}

	/* Step 8 - Display the records in use */
	puts("\n\nThe list of active records is:");
	for (j=1,i=0; j<rf->LastRec; j++)
		{
		k=rfRecStat(rf,j);
		if (k==-1)
			{
			if (rfRead(rf,j,workbuf))
				puts("ERROR: reading record");
			printf("% 4ld) %10.10s  ",j,workbuf);
			if (++i > 3)
				{
				fputc('\n',stdout);
				i=0;
				}
			}
		else if (k==-2)
			puts("ERROR: getting record status");
		}

	/* Step 6 - close the file */
	if (rfClose(rf))
		{
		puts("ERROR: can't close file");
		exit(6);
		}
	}
