/************************************************
* TONES - a set of functions that provide some	*
* interesting sonic effects.  Useful for games	*
* or alerts.									*
*												*
* 900321 MCMason - The Cobb Group, Inc.			*
************************************************/

#include <stdlib.h>
#include <dos.h>
#include "tones.h"

#define TRUE  (1==1)
#define FALSE (1==0)

/************************************************
* spTone - output a tone						*
*												*
* INP:	cyclen - Length (counts) for 1/2 cycle	*
*		numcyc - number of cycles to make		*
************************************************/
void spTone(int cyclen, int numcyc)
	{
	int t,i,j;

	numcyc <<= 1;			/* # half cycles	*/
	t=inportb(0x61);		/* Port contents	*/
	for (i=0; i<numcyc; i++)
		{
		t^=0x02;
		outportb(0x61,t);
		for (j=0; j<cyclen; j++)
			;
		}
	}


/************************************************
* spNoise - make noise for a certain amount of	*
* counts.										*
*												*
* INP:	d - the number of kilocounts of noise	*
************************************************/
void spNoise(long d)
	{
	long cnt;
	int  t,j,i;

	t=inportb(0x61);
	for (cnt=0; cnt<d; cnt += j)
		{
		j=(rand()%128)<<4;
		for (i=0; i<j; i++)
			;
		t ^= 0x02;
		outportb(0x61,t);
		}
	}

/************************************************
* spChirp - create a 'bird chirp' type noise	*
*												*
* INP:	f1 - # of counts for the starting freq.	*
*		f2 - # of counts for the ending freq.	*
*		cycles - # of cycles of each frequency	*
************************************************/
void spChirp(int f1, int f2, int cycles)
	{
	int i,j,k,l;

	l=inportb(0x61);
	cycles *= 2;
	i=f1;
	while (i != f2)
		{
		for (j=0; j<cycles; j++)
			{
			l ^= 0x02;
			outportb(0x61,l);
			for (k=0; k<i; k++)
				;
			}
		if (f1 > f2) i--;
		else		 i++;
		}
	}

/************************************************
* spSound - Generate a continuous tone using the*
* internal timer.								*
*												*
* INP:	f - the desired frequency				*
************************************************/
void spSound(unsigned f)
	{
	int c;
	if (f<19) f=19;		/* Prevent overflow */
	c = 1193180L/(long)f;
	outportb(0x43,0xb6);/* Program new divisor	*/
	outportb(0x42,c%256);/* rate into the timer	*/
	outportb(0x42,c/256);
	c=inportb(0x61);	/* Enable speaker output*/
	outportb(0x61,c|3);	/* from the timer		*/
	}

/************************************************
* spNosound - turn off the continuous tone		*
************************************************/
void spNosound(void)
	{
	int c;
	c=inportb(0x61);	/* Mask off speaker		*/
	outportb(0x61,c&0xfc);/* output from timer	*/
	}
